* mpod2_0.asm	 GP32 Monitor ROM/Programmer	Copyright 2003 Jed Margolin
***************************************************************************
* Permission is granted to individuals to make copies for their own use.
* Any commercial use requires permission of the copyright holder.
***************************************************************************
#pagelength 48t
#pagewidth 127t
$base 10t

* Uses Port E for Serial.
* Has Assembly Switches for 9600 Baud or 19.2K Baud.
* Uses 8 MHz Bus (32 MHz clock) for User Programs (9600 Baud or 19.2 KBaud).
* Security errors are flagged.
* Illegal Instructions go to SENDBREAK. 
* My Monitor works with the Debugger, but not with the Simulator.
* My Monitor code duplicate functions of Internal Monitor but uses PE0 for 
*   the Serial Port and does not require the use of other I/O pins. 
* Programming uses a 9.8304 MHz oscillator (4.9152 MHz Bus) for 19.2 KBaud, 
*   and 4.9152 MHz (2.4576 MHz Bus) for 9600 Baud.
*========================================================
* These switches control the Baud rate.
* Choose one pair or the other.
$SET 19200BAUD
$SETNOT 9600BAUD

*$SET 9600BAUD
*$SETNOT 19200BAUD
*========================================================
$INCLUDE 'gp32.h'      ; Equates for GP32
$INCLUDE 'mpod.h'
$INCLUDE 'cdrom.h'     
*========================================================
* GP20 Flash Memory is $B000-FDFF (20K Bytes)
* GP32 Flash Memory is $8000-FDFF (32K Bytes)
* My Monitor is at $FA00-$FBFF
* My utility routines (LCD) are at $F800-$F9FF
* My sample program starts at $D000
* Address table starts at $FD00 and contains pointers to:
*	 VMSG1,VMSG2, and the copyright message.
*========================================================
* These are the entry points from the routine that checks the Security Code.
*
* This is how Security seems to work:
*
*    Power-On Reset -> User Mode: All memory is accessible.
*
*    Power-On Reset -> Monitor Mode:
*	1. All memory is accessible so the Internal Monitor Code can
*	   check the Security Bytes.
*       2. If the Security Code matches, it takes the last data it received 
*	   (which would be for $FFFD) and writes it to $FFFF. All memory 
*	   remains accessible. 
*       3. If the Security Code does not match, $FFFF is read, and Security
*          Mode makes Flash Memory inaccessible. 
*
* The data written to $FFFF after a successful Security Code match may 
* be irrelevant and it is only whether the first access to $FFFF is a 
* write or a read that matters.
*
* The Security Mode remains as selected until another Power-On Reset, 
* so that a Security Code failure requires another Power-On Reset.
*
* Since my monitor runs from User Mode, Security Mode is inactive, so that
* the code cannot be secured.
*
* Unfortunately, the PE Micro Debugger checks several Security Code bytes 
* at startup. It also checks to see if the Power-On-Reset (POR) Flag is set.
* Here is the sequence obtained from capturing the Serial Port data:
*  4A  FE FB     Read $FEFB = $04  (revision number?)
*  4A  FF F6     Read $FFF6 = data (TIM1 Channel 0 high byte: a Security Byte)
*  4A  FF F7     Read $FFF7 = data (TIM1 Channel 0 low byte: a Security Byte)
*  4A  FE FB     Read $FEFB = $04  (again)
*  4A  FE 01     Read $FE01 = data (Reset Status Register; clears the I flags)
*  0C	         Read Stack Pointer
*  49  00 FE 10  Write $10 to $00FE (high byte of Return Address on Stack)
*  19  00        IWriteto $00FF (low byte of Return Address on Stack)
*  28            Run (RTI)
*
* This sends the program to $1000, which is an illegal address, causing an
* MCU Reset. It starts over, going back through Security Code verification, 
* and ends up back through this sequence. However, this time the POR Flag is 
* not set. The PE Micro software seems to want to make sure it is talking to a
* Motorola part. There are other parts in the data stream where PE Micro 
* Software deliberately causes an Illegal Instruction to be executed, thereby 
* causing an Illegal Instruction Reset.
*
* At some point the PE Micro software might read location $0040 (sflag) which
* contains information about Security. 
* sflag(7) = 1: We used parallel data input for the Security bytes
* sflag(6) = 1: Passed Security Code, 0: Failed
* 
MROM	EQU	$FA00
	ORG	MROM

START:	BRA  	REALSTART
	JMP	MYPROG		; Application Program (for easier debugging)

REALSTART:
* d5 = 1 = LVIRSTD (LVI Resets Module) disable
* d4 = 1 = LVIPWRD (LVI Module Power) disable
* d0 = 1 = COP disable
* CONFIG1 can be written to only once after a Reset. 
* In addition, d3 = LVI5OR3 (selects 3V or 5V LVI trip voltage) can be written
*   to only once after POR (Power-On Reset)
	MOV	#$31,CONFIG1	
	MOV	#$00,CONFIG2	; no SCI
	LDHX	#$0100		; locate the stack ($00FF)
	TXS
*=============================================================================
* Make sure it is set for External Clock
	BCLR	BCS,PCTL	; XCLK

	BCLR	PLLON,PCTL	; PLL off
*--------
* Initialize my I/O Ports; Monitor is using PORT E
	CLR	DDRA		; Inputs
	CLR	DDRB		; Inputs

	BCLR	0,PORTE		; d0 = SPORT
	BCLR	0,DDRE

	LDA	#$1F		; Outputs
	STA	PORTC
	STA	DDRC

	BSET	4,PDPUP		; LATCH pullup
	BSET	5,PDPUP		; LCDEN
	
	BSET	4,DDRD		; make sure LATCH is high
	BCLR	5,DDRD		; make sure LCDEN is low

	BSET	4,DDRD		; LATCH
	BSET	5,DDRD		; LCDEN
*----------------------------------
br30:	BCLR	SWEN,PORTC	 ; select switch inputs
	BRCLR	PB6,PORTA,SBYTES ; User/Dev switch
	BSET	SWEN,PORTC	; deselect switch inputs

	JMP	MYPROG		; Application Program
*---------------------------------------
* Even though we are in User Mode and Security Mode is irrelevant, there are
* some things done in Monitor Mode that the Debugger can check to see if they
* were performed so we will go through the motions of checking the Security 
* bytes and setting the appropriate flags.
SBYTES:	BSET	SWEN,PORTC	; deselect switch inputs

	AIS 	#-6		; reserve some stack space (why?)
	BCLR 	0,PORTE		; used in SENDCHAR later on
	MOV 	#$7F,sflag	; security flag: d7 = 0 
	LDHX 	#$0FFF6		; first address of security code
*----------------------------------
br31:  	CBEQX 	#$0FE,INITOUT1	; compare XReg, no flags affected
	JSR 	mGETCHAR	; Returns with a character in ACC
    	CBEQ 	X+,br31		; compare ACC to iX+, no flags affected
*----------------------------------
* something didn't match, but don't tell anyone yet
br32:  	CBEQX 	#$0FE,INITOUT2	; compare XReg, no flags affected
	JSR 	mGETCHAR	; Returns with a character in ACC
  	AIX  	#1		; no flags affected
	BRA  	br32
*----------------------------------
INITOUT1:  
	LDX  	#$0FF  		; we left Xregs at $FFFE, this makes it $FFFF
	RSP
	JMP  	ENTRY1		; we passed Security

INITOUT2:
    	BCLR 	6,sflag		; Record that Security has failed
	LDX  	#$0FF
	RSP
	JMP  	ENTRY2		; we did not pass Security

* We passed security. 
ENTRY1:	STA ,X       	

* Intruder Alert. 
ENTRY2: LDA ,X
    	BRSET 0,PORTE,br1	; not a BREAK
	JMP $0041		; BREAK, so crash and burn

br1:    SWI			; Interrupt to SWIVEC
*===============================================================
* The internal Monitor code has this at $FEFC
* It probably isn't necessary to have it this code.
XVECTORS:
	FDB	SWIVEC		; Monitor ROM SWI and Break Vector
	FDB	START		; Monitor ROM Reset Vector
*=========================================================
	ORG	MROM+$80

* Software Interrupts are produced either by the SWI instruction or by the
* Breakpoint Register. The Stack contains the information for returning to the
* User's program, or for starting it if the Stack is set up by the Host PC.
SWIVEC:	BCLR 0,DDRE		; Serial In
    	PSHH			; save HReg

* Send a Break Character to say we are ready
	BCLR 0,PORTE  		; used to make Serial Out=0
	JSR SENDBREAK	  	; ends with BRA GETCMD, ACC contains low byte 
				; of start vector = $00; PA0 is an input, 
				; pops the return address from the stack
* This is the COMMAND Interpreter
GETCMD:	JSR 	mGETCHAR	
	STA 	char		; save it

	CMP	#$4A		;			<ReadAdr>
	BNE	cmbr1a
	JSR	mReadAdr
	BRA	GETCMD	

cmbr1a:	CMP	#$49		;			<WriteAdr>
	BNE	cmbr2a
	JSR	mWriteAdr
	BRA	GETCMD	

cmbr2a:	CMP	#$1A		;			<IRead>
	BNE	cmbr3a
	JSR	mIRead
	BRA	GETCMD	

cmbr3a:	CMP	#$19		;			<IWrite>
	BNE	cmbr4a
	JSR	mIWrite
	BRA	GETCMD	

cmbr4a:	CMP	#$28		;		<Run User Program>
	BNE	cmbr5a
	JMP	mRunUser

cmbr5a:	CMP	#$0C		;			<Read SP>
	BNE	cmbr6a

ReadSP:	JSR 	mCheckBreak	; Look for BREAK character from Host PC
	TSX
    	PSHH
	PULA
	BSR SENDCHAR

	JSR	mWAIT_BIT	; saves, uses, and restores X
	JSR	mWAIT_BIT

	TXA
	BSR SENDCHAR
    	BRA GETCMD

* Illegal Instruction
cmbr6a:	JSR	SENDBREAK  	; ends with BRA GETCMD, ACC contains low byte 
				; of start vector = $00; PA0 is an input 
				; pops the return address from the stack
*----------------------------------------------------------
* Get character
* Returns with character in ACC
* If not a BREAK, automatically echoes the byte afterwards
* The simulator reports it takes $838 = 2104 cycles to get 8 bits
* 2104/8 = 104.75 us/bit. (9600 Baud is 104.16 us/bit) 
mGETCHAR:
 	BRCLR   0,PORTE,mGETCHAR   ; make sure Serial is high
gcbr1:  BRSET   0,PORTE,gcbr1      ; now wait for start bit
        BSR     mWAIT_HBIT	   ; wait 1/2 bit time

	LDA 	#$80		; this gets rotated into Carry after 8 shifts 
gcbr2:  BSR     mWAIT_BIT	; wait 1 bit time
	BRSET   0,PORTE,gcbr3   ; sets Carry according to the bit tested
gcbr3: 	RORA			; shift the Carry into the data byte
	BCC 	gcbr2		; this makes it do 8 bits

	BSR     mWAIT_BIT	; wait 1 bit time
	BRSET   0,PORTE,gcbr4   ; check for BREAK ('0' instead of a stop bit)
	JMP	SENDBREAK
* SENDBREAK ends with BRA GETCMD, ACC contains low byte of start vector = $00;
* and with PA0 is an input and pops the return address from the stack

* Continue on to echo the byte
gcbr4:	JMP	SENDCHAR
*==========================================================
*&Time
* Wait 1/2 Bit time
* called by JSR WAIT_HBIT       ;      4
mWAIT_HBIT:
        PSHX                    ;      2
*------
* 1/2 Bit at 19200 Baud = 52.083/2 = 26.041 us
* At 8 MHz, we need 26*8 = 208 cycles
*   X = 64.6; for X=64, cycles 14+3*64=206 cycles; time =206/8=25.75 us
#IF 19200BAUD
        LDX #64                 ;      2
wtrxhb: DBNZX	wtrxhb          ;               3 }loop
#ENDIF
*------
* 1/2 Bit at 9600 Baud = 104.166/2 = 52.083 us
*   cycles = 14 + 3*X ; X = (cycles-14)/3
* At 8 MHz, we need 52*8 = 416 cycles
*   X = 134; for X=100, cycles 14+3*134=416 cycles; time =416/8=52.0 us
#IF 9600BAUD
        LDX #134                ;      2
wtrxhb: DBNZX	wtrxhb          ;               3 }loop
#ENDIF
*------
        PULX                    ;      2
        RTS                     ;      4
*==========================================================
*&Time
* Wait one Bit time
* called by JSR WAIT_BIT        ;      4
mWAIT_BIT:
        PSHX                    ;      2
*------
* One Bit at 19200 Baud = 52.083 us
* At 8 MHz, we need 52.083*8 = 416.6 cycles
*   X = 100.65; for X=100, cycles 14+4*100=414 cycles; time =414/8=51.75 us
#IF 19200BAUD
        LDX #100                ;      2
wtrxb:  DECX                    ;               1 }loop
        BNE wtrxb               ;               3 }loop
#ENDIF
*------
* One Bit at 9600 Baud = 104.166 us
*   cycles = 14 + 4*X ; X = (cycles-14)/4
* At 8 MHz, we need 104.166*8 = 833 cycles
*   X = 204.75; for X=204, cycles 14+4*204=830 cycles; time =830/8=103.75 us
#IF 9600BAUD
        LDX #204                ;      2
wtrxb:  DECX                    ;               1 }loop
        BNE wtrxb               ;               3 }loop
#ENDIF
*------
        PULX                    ;      2
        RTS                     ;      4
*==========================================================
* Returns with PA0 as an output
SENDCHAR:   
	PSHX
    	PSHA
	LDA 	#$0A		; we are doing 10 (the last two are stop bits)

br22:   BRCLR 	0,PORTE,br22	; wait for Serial In to go high

	JSR 	mWAIT_BIT
	JSR 	mWAIT_BIT

    	SEC		; Carry bit "sticks" and make the stop bits
	BRA 	br24

br28:   ROR 	1,SP	; data is on the Stack			5
    	BCC 	br24	;					3
	BCLR 	0,DDRE	; Serial In (1)
	BRA 	br26

br24:   BSET 	0,DDRE	; Serial Out (0)			4
    	BRA 	br26	;					3

*&Time
*------
* 19200 Baud
* At 8 MHz we need to have a total of 416 cycles between bits, including 
*  20 cycles of overhead. (396/3=132 cycles
*  wait 3*132 cycles = 396; 396+20=416; 416/8 = 52.0 us
#IF 19200BAUD
br26:	LDX 	#132	;					2
br27:	DBNZX 	br27	;  3
#ENDIF
*------
* 9600 Baud
* At 8 MHz we need to have a total of 833 cycles between bits, including 
*  20 cycles of overhead. (813/3=271 cycles, so we will add a NOP. 
*  (813/4=203.25)
*  wait 4*203 cycles = 812; 812+20=832; 832/8 = 104.0 us
#IF 9600BAUD
br26:	LDX 	#203	;					2
br27:   NOP		;   1
	DBNZX 	br27	;   3
#ENDIF
*------
    	DBNZA 	br28	; next bit				3
	PULA
	PULX
	RTS		; returns with PA0 as an output
*==========================================================
* Check for a BREAK which might or might not happen
* we are looking for any '0's
* If there was no BREAK, it returns with Carry = 1 
* if there was a BREAK, it goes to SENDBREAK and from there to COMMAND
mCheckBreak:
        PSHX
        LDX #20		; just for grins, check every half bit time

cbr1:	JSR 	mWAIT_HBIT       
	BRCLR	0,PORTE,cbr2   
	DBNZX	cbr1

	PULX	; carry contains last state of Serial In
	RTS

cbr2:	PULX
	JMP	SENDBREAK  	; ends with BRA GETCMD, ACC contains low byte 
				; of start vector = $00; PA0 is an input 
				; pops the return address from the stack
*==========================================================================
* READ Address 	$4A: 	0 1 0 0  1 0 1 0 
* Write Address $49: 	0 1 0 0  1 0 0 1   
* IRead 	$1A: 	0 0 0 1  1 0 1 0 
* IWrite 	$19:	0 0 0 1  1 0 0 1
* ReadSP 	$0C:	0 0 0 0  1 1 0 0
* Run 		$28:	0 0 1 0  1 0 0 0
*
* CCR 			V x x H  I N Z C
spins:	BRA	spins
*==========================================================================
mReadAdr:
	JSR 	mGETCHAR
	STA 	maddress	; address high

	JSR 	mGETCHAR
	STA 	maddress+1	; address low

	JSR	mCheckBreak	; before we send it

	LDHX	maddress
	LDA	,X
	STA	mdata
	JSR	SENDCHAR

	RTS
* ---------------------------------------------------------
mWriteAdr:
	JSR 	mGETCHAR
	STA 	maddress	; address high

	JSR 	mGETCHAR
	STA 	maddress+1	; address low

	JSR 	mGETCHAR
	STA 	mdata

	JSR	mCheckBreak

	LDHX	maddress
	LDA	mdata
	STA	,X

	RTS
*---------------------
* Address has already been set up
* Send the next two bytes
mIRead:	JSR	mCheckBreak

	LDHX	maddress

	LDA 	1,X	   	; looks like we are sending something back
	JSR 	SENDCHAR

	AIX 	#2
	STHX	maddress

	LDA  	,X	
	STA	mdata
	JSR 	SENDCHAR

	RTS
*----------------
* Address has already been set up
* Write the next byte
mIWrite:
	JSR 	mGETCHAR
	STA 	mdata

	JSR	mCheckBreak

	LDHX	maddress
	AIX 	#1
	STHX	maddress

	MOV	mdata,X+
	RTS
*----------------
mRunUser:
	JSR 	mCheckBreak	; Look for BREAK character from Host PC

	PULH			; restore HReg		<RunUser>
    	RTI
*==================================================================
* Send a Break Character; 
* Goes to GETCMD with low byte of START address ($00) and with PA0 as an input
* wait for Serial In to go high, wait, 
* set Serial to Out, wait,
* set Serial to In, 
* increment SP by 2, 
* load ACC with contents of $FFFF
* goto GETCMD
SENDBREAK:
	BRCLR 0,PORTE,SENDBREAK	; wait for Serial In to go high

	JSR 	mWAIT_BIT
	JSR 	mWAIT_BIT

* Low for 10 bits (1 start bit, 8 data bits, 1 stop bit, the '1')
	LDX	#10
	BSET 	0,DDRE		; Serial Out (= '0')
br25:	JSR 	mWAIT_BIT
	DBNZX	br25

	BCLR 0,DDRE	; Serial In (makes Serial Out high)
	AIS #2		; pop the return address, we will be branching back
	LDA $FFFF	; return with low byte of START address ($00)
	JMP GETCMD
*============================================================================
* Set Clock to 8 MHz.
* Used with 32KHz clock, not used with a real clock
* Standard SCI Baud rates will no longer work
SetFastClock:

* First, change clock to XCLK
	BCLR	BCS,PCTL

* Now turn off PLL
	BCLR	PLLON,PCTL

* Wait a little
	LDX	#10
sfc1:	DBNZX	sfc1
*-------------------
* Set up PLL, using a 32.768 Xtal
* fBus = 4.9152 MHz	   fBus = 4.0 MHz	   fBus = 8.0 MHz
*      R = 1			R = 1			R = 1
*      N = $0258		N = $01E9		N = $03D1 
*      P = 0			P = 0			P = 0 
*      E = 2 			E = 1			E = 2
*      L = $80			L = $D1			L = $D0
*-------------------
* PCTL - PLL Control ->		   |  P  |     |  E  |
*	   7     6      5     4    3     2     1     0
*        PLLIE  PLLF  PLLON  BCS  PRE1  PRE0  VPR1  VPR0	
* 8MHz     0     0      0     0    0     0     1     0		$02
	MOV	#$02,PCTL	; 8 MHz
*-------------------
* PBWC - PLL Bandwidth Control (Leave Auto off)
*          7     6      5     4    3     2     1     0
*        AUTO  LOCK   /ACQ   
* 4/8MHz   0     0      0     0    0     0     0     0		$00
	MOV	#$00,PBWC	; 4/8MHz
*-------------------
* PMSH - PLL Multiplier Select High (PLL must be off)  ->N High
*          7     6      5     4    3     2     1     0
*                                  M11  M10   M9    M8 
* 8MHz     0     0      0     0    0     0     1     1		$03    ($03D1)
	MOV	#$03,PMSH	; 8 MHz
*-------------------
* PMSL - PLL Multiplier Select Low (PLL must be off)  ->N Low
*          7     6      5     4    3     2     1     0
*          M7   M6     M5    M4   M3    M2    M1    M0
*Reset     0     0      0     0    0     0     0     0		$20
* 8MHz     1     1      0     1    0     0     0     1		$D1    ($03D1)
	MOV	#$D1,PMSL	; 8 MHz
*-------------------
* PMRS - PLL VCO Range Select (PLL must be off)  ->L
*          7     6      5     4    3     2     1     0
*Reset     0     1      0     0    0     0     0     0		$20
* 8MHz     1     1      0     1    0     0     0     0		$D0
	MOV	#$D0,PMRS	; 8 MHz
*-------------------
* PMDS - PLL Reference Divider Select  ->R
*          7     6      5     4    3     2     1     0
*                                 RDS3  RDS2  RDS1  RDS0
*Reset     0     0      0     0    0     0     0     1		
* 4/8MHz   0     0      0     0    0     0     0     1		$01
	MOV	#$01,PMDS	; 4/8MHz
*-------------------
* At this point, with a 32.768 KHz xtal, the Bus clock is 8.192 KHz = 122us 
* Turn on PLL
	BSET	PLLON,PCTL

* Wait a little
	LDX	#10
sfc2:	DBNZX	sfc2

* Select the VCO Clock
	BSET	BCS,PCTL
*-------------------
* We are done
	RTS
*==================================================================
* Set Clock to Motorola's favorite speed (9.803 MHz -> 2.4576 MHz bus clock)
* Used with 32KHz clock, not used with a real clock
SetSlowClock:
	BCLR	BCS,PCTL	; First, change clock to XCLK
	BCLR	PLLON,PCTL	; Now turn off PLL

	LDX	#10		; Wait a little
ssc1:	DBNZX	ssc1

	MOV	#$01,PCTL	; PRE1,PRE0 = 0,0  VPR1,VPR0 = 0,1
	MOV	#$80,PBWC	; PLL Auto

	MOV	#$01,PMSH	; Multiplier High 
	MOV	#$2C,PMSL	; Multiplier Low

	MOV	#$80,PMRS	; VRS7..VRS0 = $80
	MOV	#$01,PMDS	; RDS3..RDS0 = $01

	BSET	PLLON,PCTL

	LDX	#10		; Wait a little
ssc2:	DBNZX	ssc2

	BSET	BCS,PCTL	; Select the VCO Clock

	RTS
*=========================================================
UNUSED	RTI	; in case we get a spurious interrupt
*=========================================================
VMSG0:  FCB  $80,'Jed',$27,'s Monitor 2.0',$FF

#IF 19200BAUD
VMSG1:  FCB  $C0,'8 MHz Bus, 19.2KBaud',$FF
#ENDIF

#IF 9600BAUD
VMSG1:  FCB  $C0,'8 MHz Bus, 9600 Baud',$FF
#ENDIF

VMSG2:  FCB  $80,'Copyright 2003',$FF
VMSG3:  FCB  $C0,' Jed Margolin ',$FF

* End of Monitor ROM
*==================================================================
*==================================================================
	ORG	VECTORS
TBASEVEC    FDB   UNUSED         ; Timebase				
ADCVEC	    FDB   UNUSED         ; A/D conversion complete		
KBVEC	    FDB   UNUSED         ; Keyboard Vector			
SCI_TX_VEC  FDB   UNUSED         ; SCI Transmit Vector			
SCI_RX_VEC  FDB   UNUSED         ; SCI Receive Vector			
SCI_ERR_VEC FDB   UNUSED         ; SCI Error Vector			
SPI_TX_VEC  FDB   UNUSED         ; SPI Transmit Vector			
SPI_RX_VEC  FDB   UNUSED         ; SPI Receive Vector			
T2_OF_VEC   FDB   UNUSED         ; Timer 2 Overflow Vector		
T2_CH1_VEC  FDB   UNUSED         ; Timer 2 Channel 1 Vector		
T2_CH0_VEC  FDB   UNUSED         ; Timer 2 Channel 0 Vector		
T1_OF_VEC   FDB   TIRQ           ; Timer 1 Overflow Vector		
T1_CH1_VEC  FDB   UNUSED         ; Timer 1 Channel 1 Vector		
T1_CH0_VEC  FDB   UNUSED         ; Timer 1 Channel 0 Vector		
PLL_VEC	    FDB   UNUSED         ; PLL Vector				
IRQ_VEC     FDB   UNUSED         ; IRQ Vector				
SWI_VEC     FDB   SWIVEC         ; Software Interrupt			
RESET_VEC   FDB   START   	 ; Beginning of program on reset	
*==========================================================================
****************************************************************************
****************************************************************************
* Programs for LCD
	ORG	$F800
****************************************************************************
*==========================================================================
* Power reset the lcd for 8-bit interface
* Comes up in 4-bit mode
pwr_lcd:
        JSR     WAIT10ms        ; wait 10 ms    <-----
        JSR     WAIT10ms        ; wait 10 ms    <-----
*--
        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT10ms        ; wait 10 ms    <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$08            ; display off
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$01            ; display on
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT10ms        ; wait 10 ms    <-----
*--
*       LDA     #$0E            ; display on, cursor on, blink off
        LDA     #$0F            ; display on, cursor on, blink on
*       LDA     #$0C            ; display on, cursor off, blink off
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$06            ; entry mode set: cursor increment, no shift
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        RTS
*==========================================================================
* Write instruction byte from accumulator during lcd powerup
lcd_pwr_instr:
	SEI			; disable interrupts
	MOV	#$FF,PORTC	; make sure IDE Interface is deselected

	MOV	#$FF,DDRA	; Command - Port A Outputs
	MOV	#$FF,DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	MOV	#$0F,PORTA	; RS, R/W = 0
	BCLR	LATCH,PORTD	
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

	CLI			; re-enable interrupts

        JSR     WAIT100us       ; <-----
        RTS
*==========================================================================
* Write instruction byte from accumulator, use Wait for Busy
lcd_wr_instr:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Command - Port A Outputs
	MOV	#$FF,DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	MOV	#$0F,PORTA	; RS, R/W = 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

	CLI			; re-enable interrupts

        BSR     WAIT_BUSY        ; <-----

        RTS
*==========================================================================
* Write Data byte from accumulator
lcd_wr_data:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Command - Port A Outputs
	MOV	#$FF,DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	MOV	#$4F,PORTA	; RS = 1, R/W = 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

	CLI			; re-enable interrupts

        BSR     WAIT_BUSY        ; <-----

        RTS
*==========================================================================
* Check the Busy Flag
WAIT_BUSY:
	JSR	WAIT100us	; just do a timed wait (100us)
	RTS
*==========================================================================
* Wait 10 us
* Need 80 machine cycles at 8 MHz
* cycles = 17 + 3 * X		X = 21 -> 80 cycles = 10.0 us
* 	JSR is 4 machine cycles		4
*&Time
WAIT10us:
        PSHX		;		2

	LDX	#21	; 		2
wt10	DBNZX	wt10    ;	3

	NOP		;		1
	NOP		;		1
	PULX		;		2
	RTS		;		4
*==========================================================================
* Display a message from absolute location in H:X
* X = #MSG  (example:   LDHX    #MSG2 )
DOMSG:  LDA     ,X             ; LCD screen address
        JSR     lcd_wr_instr

dm3lp:  AIX	#$01
        LDA     ,X
        CMPA    #$FF
        BEQ     dm3out          ; we are done

        JSR     lcd_wr_data     ; put out the character
        BRA     dm3lp

dm3out: RTS
*===========================================================================
* Wait 1 sec, then clear the display
WAITnClear:
        JSR     WAIT1S

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	RTS
*---------------------------
* Wait 2 secs, then clear the display
WAITnCLR2:
        JSR     WAIT1S
	BRA	WAITnClear
*---------------------------
* Wait 3 secs, then clear the display
WAITnCLR3:
        JSR     WAIT1S
	BRA	WAITnCLR2
*===========================================================================
* Display Binary from ACC, Xreg is cursor address
DISPLAY_BINARY:
        STA     temp
        TXA                     ; Set cursor
        JSR     lcd_wr_instr

        LDA     temp
        LSRA
        LSRA
        LSRA
        LSRA

        CMP     #$0A
        BPL     dbin1

        ADD     #$30
        BRA     dbin2

dbin1:  ADD     #$37
dbin2:  JSR     lcd_wr_data     ; first binary digit

        LDA     temp
        AND     #$0F

        CMP     #$0A
        BPL     dbin3

        ADD     #$30
        BRA     dbin4

dbin3:  ADD     #$37
dbin4:  JSR     lcd_wr_data     ; first binary digit
        RTS
*===========================================================================
* Display bcd from 0,X  ->bcdh, bcdl
* cursor from ACC
DISPLAY_BCD:
	CLRH				; we are using X as an address pointer

*       (LDA    #cursor)                ; Set cursor to first entry
        JSR     lcd_wr_instr

        LDA     1,X                     ; bcdh
        AND     #$0F
        ADD     #$30
        JSR     lcd_wr_data

        LDA    0,X                     ; bcdl
        LSRA
        LSRA
        LSRA
        LSRA
        ADD     #$30
        JSR     lcd_wr_data

        LDA     0,X                     ; bcdl
        AND     #$0F
        ADD     #$30
        JSR     lcd_wr_data

        RTS
*===========================================================================
* Display bcd from 0,X  ->bcdl
* cursor from ACC
DISPLAY_BCD2:
	CLRH				; we are using X as an address pointer

*       (LDA    #cursor)                ; Set cursor to first entry
        JSR     lcd_wr_instr

        LDA     0,X                     ; bcdl
        LSRA
        LSRA
        LSRA
        LSRA
        ADD     #$30
        JSR     lcd_wr_data

        LDA     0,X                     ; bcdl
        AND     #$0F
        ADD     #$30
        JSR     lcd_wr_data

        RTS
*===========================================================================
* Clear the Display
CLR_DISP:
        JSR     lcd_wr_instr
	LDX	#20
clrd1   LDA	#$20
	JSR     lcd_wr_data     ; put out the character
	DECX
	BNE	clrd1
	RTS
*===========================================================================
* Flash the LCD forever
FLASH_ERR:
* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Wait 1 sec
flerr:  JSR     WAIT1S

* Put out a message
        LDHX    #MSG3		; 'Error'   
        JSR     DOMSG

* Wait 1 sec
        JSR     WAIT1S

* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	BRA	flerr
*===========================================================================
* Flash the LCD once
FLASH_LCD:
* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Wait 1 sec
        JSR     WAIT1S

* Put out a message
        LDHX    #MSG2		; '   FLASH!     '   
        JSR     DOMSG

* Wait 1 sec
        JSR     WAIT1S

* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	RTS
****************************************************************************
*  Software Timing Functions
****************************************************************************
* Wait 100 us
*   cycles = 15 + 4*X ; X = (cycles-15)/4
*
* At 8 MHz, we need 100*8 = 800 cycles
*   X = 196.25; for X=196, cycles 15+4*196=799 cycles; time =799/8=99.8 us
*&Time
*       JSR WAIT_BIT  		;       4
WAIT100us:
        PSHX			;	2

	LDX	#196		; 	3
wt100	NOP			; } 1
	DBNZX	wt100   	; } 3

	PULX			;	2
	RTS			;	4
*--------------------------------------------------------------------------*
WAIT200us:
        BSR	WAIT100us
        BSR	WAIT100us
	RTS
*---------------------------------------------------------
* Wait 1000 us
*   cycles = 19 + 8*X ; X = (cycles-19)/8
*
* At 8 MHz, we need 1000*8 = 8000 cycles
*  X = 997.625; for X=997, cycles 19+8*997=7995 cycles; time =7995/8=999.375us
*&Time
* 	JSR is 4 machine cycles	4
WAIT1ms:
        PSHH		;	2
        PSHX		;	2

	LDHX	#997	;	3

wt1ms	AIX	#-1	; } 2
	CPHX	#0000	; } 3
	BNE	wt1ms   ; } 3

	PULH		;	2
	PULX		;	2
	RTS		;	4
*---------------------------------------------------------
WAIT2ms:
        BSR     WAIT1ms
        BSR     WAIT1ms
        RTS
*---------------------------------------------------------
WAIT4ms:
	PSHX

	LDX	#4
wt4lp:	BSR	WAIT1ms
	DBNZX	wt4lp

	PULX
	RTS
*---------------------------------------------------------
WAIT10ms:
	PSHX

	LDX	#10
wt10lp:	BSR	WAIT1ms
	DBNZX	wt10lp

	PULX
	RTS
*---------------------------------------------------------
WAIT100ms:
	PSHX

	LDX	#10
wt100lp:
	BSR	WAIT1ms
	DBNZX	wt100lp

	PULX
	RTS
*---------------------------------------------------------
WAIT250ms:
	PSHX

	LDX	#250
wt250lp:
	BSR	WAIT1ms
	DBNZX	wt250lp

	PULX
	RTS
*---------------------------------------------------------
* Wait 1 second
WAIT1s:
	PSHX

	LDX	#100
wt1slp:
	BSR	WAIT10ms
	DBNZX	wt1slp

	PULX
	RTS
*==========================================================================
	ORG	$FD00
ADRTABLE:	FDB	VMSG0,VMSG1,VMSG2,VMSG3
*==========================================================================
* Start of User Program
	ORG	$D000

Time_Test:
* Initialize my I/O Ports 
* Monitor is using PORT D
	CLR	DDRA		; Inputs
	CLR	DDRB		; Inputs

	BCLR	0,PORTE		; d0 = SPORT
	BCLR	0,DDRE

	LDA	#$1F		; Outputs
	STA	PORTC
	STA	DDRC

	BSET	4,PDPUP		; LATCH pullup
	BSET	5,PDPUP		; LCDEN
	
	BSET	4,DDRD		; make sure LATCH is high
	BCLR	5,DDRD		; make sure LCDEN is low

	BSET	4,DDRD		; LATCH
	BSET	5,DDRD		; LCDEN

	JSR	pwr_lcd		; power-reset the lcd

* Application Program
        LDHX    #VMSG0 		; 'My Monitor x.x'
        JSR     DOMSG

        LDHX    #VMSG1 		; 'x MHz Bus, x Baud'
        JSR     DOMSG

	BSET	0,DDRD

ploop1:	BSET	0,PORTD		; toggle PD0
	JSR	WAIT1ms

	BCLR	0,PORTD
	JSR	WAIT1ms

  	BCLR	SWEN,PORTC	 	; select switch inputs

	BRCLR	PB5,PORTA,DBREAK	; Port A inputs (Switch 5)

	BSET	SWEN,PORTC		; deselect switch inputs
	BRA	ploop1

* Set Debugger Break here to end the loop
DBREAK:	BSET	SWEN,PORTC		; deselect switch inputs
	RTS
*==========================================================================
* Display the Security Code
SCode:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

        LDHX    #MMSG4 			; '$FFF6-$FFFD'
        JSR     DOMSG

	MOV	#$FF,daddress
	MOV	#$F6,daddress+1

	MOV	#$8D,lcdadr		; display address
	MOV	#2,bcount

	BSR	Display_Data

	MOV	#$FF,daddress
	MOV	#$F8,daddress+1

	MOV	#$C1,lcdadr		; display address
	MOV	#6,bcount

	BSR	Display_Data

scodbr:	BRA	scodbr
*--------------------------------------
Display_Data:
ddbr:	LDHX	daddress		; address of data
	LDA	,X			; data
	AIX	#1
	STHX	daddress

	LDX	lcdadr			; display address	
        JSR     DISPLAY_BINARY  	; Display Binary from ACC

	LDA	lcdadr			; advance the lcd display address
	ADD	#3
	STA	lcdadr

	DEC	bcount
	BNE	ddbr
	RTS
*==========================================================================
pMSG0:  FCB  $80,'Break',$FF
pMSG1:  FCB  $80,'Ready',$FF
pMSG2:  FCB  $80,'   FLASH!    ',$FF
pMSG3:  FCB  $80,'Error',$FF
pMSG4:  FCB  $80,'Monitor Mode ',$FF
pMSG5:  FCB  $80,'My Program   ',$FF
*==========================================================================
* Start of CDROM Program
* Standalone Controller for an IDE CD-ROM Drive    Rev 2.0   2/03
	ORG	$B000

MYPROG:
* Initialize my I/O Ports 
* Monitor is using PORT D
	CLR	DDRA		; Inputs
	CLR	DDRB		; Inputs

	BCLR	0,PORTE		; d0 = SPORT
	BCLR	0,DDRE

	LDA	#$1F		; Outputs
	STA	PORTC
	STA	DDRC

	BSET	4,PDPUP		; LATCH pullup
	BSET	5,PDPUP		; LCDEN
	
	BSET	4,DDRD		; make sure LATCH is high
	BCLR	5,DDRD		; make sure LCDEN is low

	BSET	4,DDRD		; LATCH
	BSET	5,DDRD		; LCDEN

	JSR	pwr_lcd		; power-reset the lcd
*=============================================================================
* Do a hardware reset of the CDROM Drive.
* This should be the only place that does a hardware reset.
* Note that if the tray is open, a hardware reset will close it.
* This can be disconcerting if it is not expected.
	JSR	Hard_Reset

*   Give a menu to select: 1) Play CD; 2) Drive Info; 3) SCode; 4)Timing Test
PSTART: BSET	SWEN,PORTC	; deselect switches
	JSR	Display_Men_Msgs

* Read switches until one is selected
smlp:	BCLR	SWEN,PORTC	; select switch inputs
	BRCLR	PB0,PORTA,smbr1 ; Port A inputs
	BRCLR	PB1,PORTA,smbr2 ; Port A inputs
	BRCLR	PB2,PORTA,smbr3 ; Port A inputs
	BRCLR	PB3,PORTA,smbr4 ; Port A inputs
	BRA	smlp

smbr1:  BSET	SWEN,PORTC	; deselect switch inputs
	JMP	PLAY_CD		; PLAY CD

smbr2:  BSET	SWEN,PORTC	; deselect switch inputs
	JSR	Drive_Info	; Drive Info
	JSR	WAIT1s
	BRA	PSTART

smbr3:  BSET	SWEN,PORTC	; deselect switch inputs
	JSR	SCode		; Security Code
	BRA	PSTART

smbr4:  BSET	SWEN,PORTC	; deselect switch inputs
	JSR	Time_Test	; Timing Test
	BRA	PSTART
*=============================================================================
****************************************************************************
*  CD Player
****************************************************************************
*=============================================================================
PLAY_CD:

* Software Reset
	JSR	Soft_Reset

*  Put the address of the Main Timer Overflow function into the Vector 
*  Indirection Table.
	CLR	irqflag

	SEI
	JSR	INIT_SWITCHES

* Set up the Timer Interrupt 
	MOV	#$26,T1MODH	; must write to high byte first
	MOV	#$00,T1MODL

	MOV	#$01,T1SC	; PS2, PS1, PS0; TSTOP = 0

	BSET	TOIE,T1SC	; enable Timer interrupts

	CLI			; Enable Interrupts

* Clear the display
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Wait_For_Ready
mcdbr1:	JSR	Wait_For_Ready
	BEQ	mcdbr2
	JMP	FLASH_ERR

* Start the Disc
mcdbr2:	JSR	Start_Disc
	BEQ	mcdbr3
	JMP	FLASH_ERR

* Wait_For_Ready
mcdbr3:	JSR	Wait_For_Ready
	BEQ	mcdbr4
	JMP	FLASH_ERR

mcdbr4:	JSR	Make_TOC10	; make the Table of Contents

* Clear the display
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	Time_Header		; Display Header for SubCode time

* Display the number of tracks on the disc
	LDA	track_count
	JSR	BIN_BCD_99	; convert to BCD

	LDA	bcd0
        LDX     #$D2    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

* Initialize the track parameters
	LDA	#$01
	STA	track
	STA	newtrack

* Read the SubCode Time and Track
	JSR	Read_SubCode		; Track is in buffer+6
	LDA	buffer+6
	STA	track
	CLRH

* Display it
	JSR	Show_Time

	CLR	playmode
	CLR	chtrack

	LDA	#$01
	STA	track
	STA	newtrack
*------------------------------------------------------------------------
* CD Player Command Function
CD_Command:

cdlp:	

* if status(d0) [check] is set it probably means the tray has been opened
	JSR	Get_Alt_Status
	BRCLR	0,status,cdbr1

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	Wait_For_Ready		; wait for disc to be installed

	JSR	Soft_Reset		; software reset

	JSR	Start_Disc		; start the disc

	JSR	Make_TOC10		; Make TOC Table

	JSR	Time_Header		; Display Header for SubCode time

	LDA	track_count	; Display the number of tracks on the disc
	JSR	BIN_BCD_99	; convert to BCD
	LDA	bcd0
        LDX     #$D2    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	#$01
	STA	track
	STA	newtrack
	CLR	chtrack
	JSR	Play_Track
*---------------------------
* if sw5 and sw6 are both pressed, just stop, so we can reset it 
* without screwing up the TOC Table
* In swx+1, d0 is current debounced state. Pressed = 0
cdbr1:	BRSET	0,sw5+1,cdbr2
	BRSET	0,sw6+1,cdbr2

	BCLR	2,sw5+1
	BCLR	2,sw6+1

	SEI			; disable interrupts

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	LDHX	#IMSG4		; 'Program Halted'
	JSR     DOMSG

spin:	BRA	spin
*---------------------------
* Stop Switch
cdbr2:	BRCLR	2,sw2+1,cdbr3		; no
	BCLR	2,sw2+1			; yes, clear the flag

	CLR	playmode

	JSR	Do_STOP

	BRA	cdlp
*---------------------------
* Play Switch
cdbr3:	BRCLR	2,sw1+1,cdbr4		; no
	BCLR	2,sw1+1			; yes, clear the flag

* If it is already playing, don't start it
	BRSET	7,playmode,cdbr4	; continue

* Otherwise, start the track
	LDA	newtrack
	JSR	Play_Track

	BCLR	7,playmode

	BRA	cdlp
*---------------------------
* Read the SubCode Time and Track
cdbr4: 	JSR	Read_SubCode		; Track is in buffer+6
	LDA	buffer+6
	STA	track

	CLRH

* Display it
	JSR	Show_Time
*---------------------------
* Swtches are in the Interrupt routine
* Do we have a new track?
	BRSET	7,chtrack,cdbr5		; yes

* If we don't have a change of track, the disc may have played past the
* currently selected track, so update the newtrack
	LDA	track
	STA	newtrack
	JSR	BIN_BCD_99	; convert to BCD

	LDA	bcd0
        LDX     #$92    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	JMP	cdlp

* The track has been changed, so update the track and play it
cdbr5:	BCLR	7,chtrack	; clear it

	LDA	newtrack
	STA	track
	JSR	BIN_BCD_99

	LDA	bcd0
        LDX     #$92    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	newtrack
	JSR	Play_Track

	JMP	cdlp
*=============================================================================
****************************************************************************
* Functions for the CDROM Drive
****************************************************************************
*==========================================================================
* CDROM Drive Hardware Reset
Hard_Reset:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F7,PORTA
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; Reset Low

	JSR	WAIT100us

	MOV	#$FF,PORTA
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; Reset high

	CLR	DDRA		; Port A Input

	CLI			; re-enable interrupts

	RTS
*==========================================================================
*==========================================================================
* CDROM Drive Software Reset
Soft_Reset:

* Give Busy a chance to go low
	LDA	#$FF
	STA	pcount
sr1:	LDA	#$07			; read real status register
	STA	PortAddress
	JSR	ReadPort
	STA	status

	BRCLR	7,status,sr2		; Busy must be Low
	JSR	WAIT10ms
	DEC	pcount
	BNE	sr1

	LDA	#$FF			; error (status valid)
	RTS

* Write the command
sr2:	LDA	#$00			; device 0
	STA	datasave
	LDA	#DRIVE_SEL
	JSR	WritePort

	LDA	#SOFT_RESET_CMD
	STA	cmdsave
	JSR	Write_Command		; from cmdsave

* Give Busy a chance to go low
	LDA	#$FF
	STA	pcount
sr3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,srout		; Busy must be Low
	JSR	WAIT10ms
	DEC	pcount
	BNE	sr3

	LDA	#$FF			; error (status valid)
	RTS

* status will be $00 until the first ATAPI command
* Do a Test_Unit_Ready (Get_Sense)
srout:	JSR	Get_Sense

	LDA	sense_error
	STA	status

	RTS
*============================================================================
*============================================================================
Show_Status:
	STX	binl

	LDA	#$07
	STA	PortAddress
	JSR	ReadPort
	STA	status
*	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC

	CLRH
        LDX     binl    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC
	RTS
*==========================================================================
*===========================================================================
* Get and display the Drive Information 
* Uses 'Drive Inquiry' and 'Read Drive Capabilities'
Drive_Info:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	LDHX	#DIMSG0		; 'Drive Info'
	JSR	DOMSG

	JSR	WAIT1S
	JSR	WAIT1S

	JSR	Soft_Reset

* Inquiry
	CLRH
	JSR	Do_Inquiry
	BEQ	dibr1
	LDA	#$01			; error
	RTS

* Drive Capabilities
dibr1:	JSR	Get_Sense

	CLRH
	JSR	Drive_Caps_Cmd		; Get Drive Capabilities
	BEQ	dibr2
	LDA	#$02			; error
	RTS

* Show me the data
dibr2:	JSR	Show_RDC_All
	CLRA				; success
	RTS
*-----------------
DIMSG0:  FCB  $80,'Drive Info',$FF
DIMSG1:  FCB  $80,'Done',$FF
DIMSG2:  FCB  $80,'Error:',$FF
IMSG4:   FCB  $83,'Program Halted ',$FF
*===========================================================================
*==========================================================================
* Start_Disc Packet Command
Start_Disc:
	CLR	sense_error

* Form the command packet for Start Disc
	CLRA
	LDHX	#11			; X= 11->0 is packet[0..11]
sd1:	   STA	  buffer,X
	   DECX
	   BPL    sd1
      LDA	#START_STOP_EJECT
      STA	buffer+11

      MOV	#$01,buffer+7		; start the disc and read the TOC

      JSR	Do_Packet		; errlog in ACC
      
      RTS
*==========================================================================
*==========================================================================
Play_Track:
EndM	EQU	99	; End M
EndS	EQU	59	; End S
EndF	EQU	74	; End F

* Play Audio MSF from Track Number in ACC
Play_MSF:

* Get the MSF data from the TOC table
* Track number in ACC
	DECA			; track 1 = ptr 0
	LDX	#3		; 3 entries/track
	MUL			; low 8 bits in ACC
	TAX
	CLRH			; we are using X as an address pointer

* Get MSF
	LDA	cbuffer,X
	STA	Mval

	INCX
	LDA	cbuffer,X
	STA	Sval

	INCX
	LDA	cbuffer,X
	STA	Fval

* Form the command packet for PLAY_AUDIO_MSF
        MOV	#PLAY_AUDIO_MSF,buffer+11	; byte 0	Op Code
	CLR	buffer+10	; byte 1
	CLR	buffer+9	; byte 2
	MOV	Mval,buffer+8	; byte 3	Start M
	MOV	Sval,buffer+7	; byte 4	Start S
	MOV	Fval,buffer+6	; byte 5	Start F
	MOV	#EndM,buffer+5	; byte 6	Ending M
	MOV	#EndS,buffer+4	; byte 7	Ending S
	MOV	#EndF,buffer+3	; byte 8	Ending F
	CLR	buffer+2	; byte 9
	CLR	buffer+1	; byte 10
	CLR	buffer+0	; byte 11

	JSR	Do_Packet	; returns $00 = no errors
	BEQ	pmout
	JSR	FLASH_LCD	; flash it once

* done
pmout:	RTS
*==========================================================================
*==========================================================================
TOC10:	FCB   $00,$00		; 11,10
	FCB   $80,$00		; 9,8
	FCB   $20,$01		; 7,6
	FCB   $00,$00		; 5,4
	FCB   $00,$10		; 3,2
	FCB   $02,READ_TOC	; 1,0

* Read Table of Contents 10 (for audio CD tracks)
Make_TOC10:

	CLR	track_count
	CLR	track_ptr

* Form the command packet for Read TOC 10
	LDHX	#11			; X= 11->0 is packet[0..11]
toc1:	   LDA	  TOC10,X
	   STA	  buffer,X
	   DECX
	   BPL    toc1

      JSR	Do_Packet		; returns $00 = no errors
      BEQ	toc2
      JMP	FLASH_ERR

* wait while BSY
toc2:

* get and decode the data
	JSR     READ_CD_WORD	; length

	MOV	byte_low,length+1	; length MSB
	MOV	byte_high,length+0	; length LSB

	LDA	length+0	; subtract 4+11
	ADD	#$F1
	STA	length+0

	LDA	length+1
	ADC	#$FF
	STA	length+1

	JSR     READ_CD_WORD	; first, last session (ignore)

* There are 11 bytes/record
toclp:	JSR     READ_CD_WORD	; Session #, ADR:Control (ignore)

	JSR     READ_CD_WORD	; TNO (ignore), Point

	LDA	byte_high	; 0-99 is track number
	STA	track
	CMP	#99
	BMI	toc3	

* not a track number 
	JSR     READ_CD_WORD	; byte 4, 5
	JSR     READ_CD_WORD	; byte 6, 7
	JSR     READ_CD_WORD	; byte 8, 9 
	JSR     READ_CD_WORD    ; byte 10, 0 (byte_high is byte 0 next record)
	BRA	toc_odd

* we have a track number
toc3:
	JSR     READ_CD_WORD	; byte 3, byte 4 (ignore)
	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; M, S
	MOV	byte_low,Mval
	MOV	byte_high,Sval

	JSR     READ_CD_WORD	; F, first bute of next record
	MOV	byte_low,Fval

	JSR	Log_MSF

* byte_high is first byte of next record
toc_odd:
	JSR	Do_Length
	BPL	toc4		; msb less than zero means we are done

* we are done, so just read and discard 2048 bytes
	JSR	FLUSH
	RTS

* byte_high is byte 0 of next record, ignore it
toc4:	JSR     READ_CD_WORD    ; byte 1, 2 (ignore)

	JSR     READ_CD_WORD	; byte 3 Point, byte 4 (ignore)

	LDA	byte_low	; byte 3: 0-99 is track number
	STA	track
	CMP	#99
	BMI	toc5	

* not a track number 
	JSR     READ_CD_WORD	; byte 5, byte 6
	JSR     READ_CD_WORD	; byte 7, byte 8
	JSR     READ_CD_WORD	; byte 9, byte 10
	JMP	toclp		; we are at an even boundary again

toc5:	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; byte 7, byte 8 (Mvalue)
	MOV	byte_high,Mval

	JSR     READ_CD_WORD	; S, F
	MOV	byte_low,Sval
	MOV	byte_high,Fval

	JSR	Log_MSF

	JSR	Do_Length
	BMI	toc6		; msb less than zero means we are done
	JMP	toclp

*  we will just read and discard 2048 bytes
toc6:	JSR	FLUSH
	RTS
*==========================================================================
* Add the MSF data to the buffer (for making TOC table)
Log_MSF:
	CLRH			; we are using X as an address pointer
	LDX	track_ptr

	LDA	Mval
	STA	cbuffer,X
	INCX

	LDA	Sval
	STA	cbuffer,X
	INCX

	LDA	Fval
	STA	cbuffer,X
	INCX

	STX	track_ptr

	INC	track_count

	RTS
*==========================================================================
*===========================================================================
* Get Drive Capabilities, return data in cbuffer
Drive_Caps_Cmd:

* Make the packet
	JSR	Get_Caps

* Send Packet Command
	JSR	Packet_Command		; send Packet Command
	BEQ	drcbr1

	LDA	#$01			; Busy never went low
	RTS

* Give DRQ a chance to go high
drcbr1:	MOV	#$FF,pcount
drclp2:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,drcbr2		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	drclp2

	LDA	#$02			; DRQ never went high
	RTS

* Send the Packet
drcbr2:	JSR	Write_DBuffer   	; returns status in ACC

* Give Busy a chance to go low
	MOV	#$FF,pcount
drcct1:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,drcct2		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	drcct1

	LDA	#$03			; Busy never went low
	RTS

drcct2:	

* Give DRQ a chance to go high
	MOV	#$FF,pcount
drcct3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,drcct4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	drcct3

	LDA	#$04		; DRQ did not go high
	RTS

* Get the data (20 bytes: Goes into cbuffer)
drcct4:	JSR	GCDATA
	BEQ	drcct5

	LDA	#$06		; bad news, no data
	RTS

drcct5: CLRA			; success
	RTS
*==========================================================================
*==========================================================================
RDC:	FCB   $00,$00		  ; 11,10
	FCB   $00,$20		  ; 9,8   0,Length LSB
	FCB   $00,$00		  ; 7,6   
	FCB   $00,$00		  ; 5,4   
	FCB   $00,$AA		  ; 3,2   0, PC/Page Code
	FCB   $00,MODE_SENSE      ; 1,0

* Make Command Buffer for 'Read Drive Capabilities'
Get_Caps:
	LDHX	#11			; X= 11->0 is packet[0..11]
rdc1:	   LDA	  RDC,X
	   STA	  buffer,X
	   DECX
	   BPL    rdc1
       RTS
*==========================================================================
* Get the data from 'Read Drive Capabilities'
GCDATA:	LDX	#100		; timeout if we don't find the real data
	
gcd1:	JSR	READ_CD_WORD

	LDA	#$2A		; Real data starts with $2A

	CMP	byte_low
	BEQ	gcd2

	CMP	byte_high
	BEQ	gcd4

	DECX
	BNE	gcd1

	LDA	#10		; didn't get it
	RTS

* started with low byte
gcd2:	CLRX
	CLRH

	LDA	byte_low
	STA	cbuffer,X
	INCX

	LDA	byte_high
	STA	cbuffer,X
	INCX

* do 9 more words (18 bytes)
	MOV	#9,vcount

gcd3:	JSR	READ_CD_WORD

	LDA	byte_low
	STA	cbuffer,X
	INCX

	LDA	byte_high
	STA	cbuffer,X
	INCX

	DEC	vcount
	BNE	gcd3

	RTS			; returns with ACC=0 (success)

* started with high byte
gcd4:	CLRX
	CLRH

	LDA	byte_high
	STA	cbuffer,X
	INCX

* do 9 more words (18 bytes)
	MOV	#9,vcount

gcd5:	JSR	READ_CD_WORD

	LDA	byte_low
	STA	cbuffer,X
	INCX

	LDA	byte_high
	STA	cbuffer,X
	INCX

	DEC	vcount
	BNE	gcd5

	JSR	READ_CD_WORD	; one more byte

	LDA	byte_low
	STA	cbuffer,X

	CLRA
	RTS			; returns with ACC=0 (success)
*==========================================================================
* Messages for Drive Info
DMSG0:  FCB  $C0,'CD-R Read:',$FF
DMSG1:  FCB  $C0,'CD-RW Read:',$FF

DMSG2:  FCB  $C0,'CD-R Write:',$FF	
DMSG3:  FCB  $C0,'CD-RW Write',$FF	

DMSG4:  FCB  $C0,'Mode 2 Form 1:',$FF
DMSG5:  FCB  $C0,'Mode 2 Form 2:',$FF
DMSG6:  FCB  $C0,'Multi Session:',$FF	
DMSG7:  FCB  $C0,'Audio Play:',$FF	

DMSG8:  FCB  $CF,'Yes',$FF
DMSG9:  FCB  $CF,'No',$FF

DMSG10:  FCB  $83,'Max Speed:',$FF	
DMSG11:  FCB  $C8,'KB/sec',$FF		
DMSG12:  FCB  $83,'Buffer Size:',$FF	
DMSG13:  FCB  $CA,'KBytes',$FF		

DMSG14:  FCB  $CE,'Done',$FF
*--------------------------------
* Show some Read_Drive_Capabilities
Show_RDC:

	CLR	pcount

* 'CD-R Read'
	JSR	Show_Info	; pcount comes back incremented
	JSR	WAITnCLR2	; wait 2 seconds, then clear the display

* 'CD-RW Read'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-R Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-RW Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 1'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 2'
	JSR	Show_Info		
	JSR	WAITnCLR2
	
* 'Multi Session'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Audio Play'	
	JSR	Show_Info		
	JSR	WAITnCLR2

	RTS			; we are done	
*--------------------------------
* Show Read_Drive_Capabilities
Show_RDC_All:

	CLR	pcount

* 'CD-R Read'
	JSR	Show_Info	; pcount comes back incremented
	JSR	WAITnCLR2	; wait 2 seconds, then clear the display

* 'CD-RW Read'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-R Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-RW Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 1'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 2'
	JSR	Show_Info		
	JSR	WAITnCLR2
	
* 'Multi Session'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Audio Play'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Max Speed'	
	JSR	Show_Speed	; Show the Speed data
	JSR	WAITnCLR2	

* 'Buffer Size'
	JSR	Show_Bsize	; Show the Buffer Size data
	JSR	WAITnCLR2	

	RTS			; we are done	
*==========================================================================
*		Message (Word), Data Address (Word), Mask (Byte), 

Info_Table:   
      FDB	DMSG0,cbuffer+2  
      FCB	$01		; CD-R Read

      FDB	DMSG1,cbuffer+2  
      FCB	$02		; CD-RW Read

      FDB	DMSG2,cbuffer+3  
      FCB	$01		; CD-R Write

      FDB	DMSG3,cbuffer+3  
      FCB	$02		; CD-RW Write

      FDB	DMSG4,cbuffer+4  
      FCB	$10		; CD-R Write

      FDB	DMSG5,cbuffer+4  
      FCB	$20		; CD-RW Write

      FDB	DMSG6,cbuffer+4  
      FCB	$10		; CD-R Write

      FDB	DMSG7,cbuffer+4  
      FCB	$20		; CD-RW Write
*------------------------------------
* Do the Drive Info_Table
* pcount is index to Info_Table, comes back incremented
Show_Info:

* get the offset of the table entry
	LDX	pcount
	LDA	#5		; 5 bytes/entry
	MUL
	TAX

	CLRH

* address of message
	LDA	Info_Table+0,X
	STA	address1+0	; msb

	LDA	Info_Table+1,X
	STA	address1+1	; lsb

* address of buffer data
	LDA	Info_Table+2,X
	STA	address2+0	; msb

	LDA	Info_Table+3,X
	STA	address2+1	; lsb

* mask
	LDA	Info_Table+4,X
	STA	mask

* Do the message
	LDHX	address1	; address of table entry
	JSR	DOMSG

* Get the data and use the data mask
	LDHX	address2	; address of buffer data
	LDA	,X   		; buffer data
	AND	mask
	JSR	YesNo

	INC	pcount
	RTS
*----------------------------
* Display 'yes' or 'no' depending in Z Flag
* Zero Flag: 1 = yes, 0 = no
YesNo:	BEQ	no

yes:	LDHX	#DMSG8		; yes
	BRA	ynbr

no:	LDHX	#DMSG9		; no

ynbr:   JSR     DOMSG

	RTS
*----------------------------
* Show the Speed data
Show_Speed
	LDHX	#DMSG10		; Speed
	JSR     DOMSG

	LDA	cbuffer+8	; msb
	STA	binh
	LDA	cbuffer+9	; lsb
	STA	binl

	JSR	Bin_6BCD	; convert 16 bit binary to 5 BCD digits
	JSR	Display_6BCD

	LDHX	#DMSG11		; KB/secs
	JSR     DOMSG

	RTS
*----------------------------
* Show the Buffer Size data
Show_BSize
	LDHX	#DMSG12		; Speed
	JSR     DOMSG

	LDA	cbuffer+12	; msb
	STA	binh
	LDA	cbuffer+13	; lsb
	STA	binl

	JSR	Bin_6BCD	; convert 16 bit binary to BCD
	JSR	Display_6BCD

	LDHX	#DMSG13		; KB/secs
	JSR     DOMSG

	RTS
*==========================================================================
*==========================================================================
INQ:	FCB   $00,$00		; 11,10
	FCB   $00,$00		; 9,8
	FCB   $00,$00		; 7,6
	FCB   $00,96		; 5,4    4=allocation
	FCB   $00,$00		; 3,2
	FCB   $00,INQUIRY	; 1,0

* Do the Inquiry command and report results
Do_Inquiry:

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Form the command packet for Read TOC 10
	CLRH
	LDHX	#11			; X= 11->0 is packet[0..11]
inq1:	   LDA	  INQ,X
	   STA	  buffer,X
	   DECX
	   BPL    inq1

      JSR	Do_Packet		; returns $00 = no errors
      BEQ	inq2
      JMP	FLASH_ERR

* wait while BSY
inq2:

* get and decode the data
	JSR     READ_CD_WORD	; pass
	JSR     READ_CD_WORD	; pass
	JSR     READ_CD_WORD	; pass
	JSR     READ_CD_WORD	; pass

* Vendor
	LDHX	#QMSG0		; Vendor
	JSR     DOMSG

	LDA	#$C0		; start of second line
        JSR     lcd_wr_instr

	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word

	JSR	WAITnCLR3

* Product
	LDHX	#QMSG1		; Model
	JSR     DOMSG

	LDA	#$C0	; start of second line
        JSR     lcd_wr_instr

	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word

	JSR	WAITnCLR3

* Product Rev Level
	LDHX	#QMSG2		; Rev
	JSR     DOMSG

	LDA	#$85	
        JSR     lcd_wr_instr

	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word

	JSR	WAITnCLR3

	JSR	FLUSH

	RTS
*==========================================================================
* Read and display a word
RnD:	JSR     READ_CD_WORD

	LDA	byte_low
	JSR	lcd_wr_data

	LDA	byte_high
	JSR	lcd_wr_data

	RTS
*------------------------------------
QMSG0:  FCB  $80,'Vendor:',$FF
QMSG1:  FCB  $80,'Model:',$FF
QMSG2:  FCB  $80,'Rev:',$FF
*==========================================================================
* Time comes from Reading the SubCode
Show_Time:
	LDA	buffer+6		; Track
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$86
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+13		; Relative M
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$8B
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+14		; Relative S
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$8E
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+9		; Absolute M
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$CB
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+10		; Absolute S
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$CE
	CLRH
	JSR	DISPLAY_BINARY

	RTS
*==========================================================================
Time_Header:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	LDHX	#TMSG0		; 'Track:'
        JSR     DOMSG

	LDHX	#TMSG1		; 'Abs Time:'
        JSR     DOMSG

	LDA	#$8D
        JSR     lcd_wr_instr
	LDA	#':'
        JSR     lcd_wr_data

	LDA	#$CD
        JSR     lcd_wr_instr
	LDA	#':'
        JSR     lcd_wr_data

	RTS

TMSG0:  FCB  $80,'Track:',$FF
TMSG1:  FCB  $C0,'Abs Time:',$FF
*==========================================================================
* Read SubCode to get the current time
Read_SubCode:
	JSR	Make_Sub_Packet		; Make SubCode Command Packet

* Send Packet Command
	JSR	Do_Packet
	BEQ	rscbr1

	LDA	#$01		; Busy never went low
	RTS

* Get the data
rscbr1:	JSR	Read_Sub_Data
	RTS
*==========================================================================
*==========================================================================
SUB:	FCB   $00,$00		  ; 11,10
	FCB   $00,$10		  ; 9,8   Length MSB,LSB
	FCB   $00,$00		  ; 7,6   
	FCB   $00,$00		  ; 5,4   
	FCB   $01,$40		  ; 3,2   Format, $40
	FCB   $02,SUBCHAN         ; 1,0   MSF,Cmd

* Make Command Packet for 'Read Sub Channel'
Make_Sub_Packet:
	LDHX	#11			; X= 11->0 is packet[0..11]
gsp1:	   LDA	  SUB,X
	   STA	  buffer,X
	   DECX
	   BPL    gsp1
       RTS
*==========================================================================
* Get the data from 'Read SubChannel'
Read_Sub_Data:

	CLRX
	CLRH

* do 8 words (16 bytes)
	MOV	#8,vcount

rsb1:	JSR	READ_CD_WORD

	LDA	byte_low
	STA	buffer,X
	INCX

	LDA	byte_high
	STA	buffer,X
	INCX

	DEC	vcount
	BNE	rsb1

	RTS
*==========================================================================
* Do the Stop command
Do_STOP:

* Form the command packet for Read TOC 10
	LDHX	#11			; X= 11->0 is packet[0..11]
stp1:	   LDA	  #00
	   STA	  buffer,X
	   DECX
	   BPL    stp1

      LDA	#STOP
      STA	buffer+11

      JSR	Do_Packet		; returns $00 = no errors
      RTS
*==========================================================================
* send Packet Command
* If ACC returns with $FF, BUSY never went low
* Otherwise, ACC returns $00, with status register in status
Packet_Command:

* Give Busy a chance to go low
	MOV	#$FF,pcount
ct3a:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,ct3b		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	ct3a

	LDA	#$FF			; Busy never went low
	RTS

* Write the command
ct3b:	MOV	#ATAPI_PACKET_CMD,cmdsave
	JSR	Write_Command		; from cmdsave

* read the real status register (resets INTRQ)
	MOV	#$07,PortAddress
	JSR	ReadPort
	STA	status
	CLRA
	RTS
*==========================================================================
* Send 'Packet Command' and Packet Data from buffer
* Returns in ACC (also errlog):
*   $00    No Errors
*   $FE    Before 'Send Packet Command' BUSY never went low, command not sent
*   $FD    After 'Send Packet Command' DRQ never went high
*   $FC    After sending Packet data BUSY never went low
*
Do_Packet:
	CLR	errlog

* Send Packet Command
*    If ACC returns with $01, BUSY never went low
*    Otherwise, ACC returns $00, with status register in status
	JSR	Packet_Command	     ; send Packet Command
	BEQ	dp2
	LDA	#$01
	STA	errlog
	RTS		        ; $01 = BUSY never went low, command not sent

* Packet command was sent

* Give DRQ a chance to go high
dp2:	LDA	#$FF
	STA	pcount
dp3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,dp4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	dp3

	LDA	#$02			; DRQ never went high
	RTS

* Send the Packet
dp4:	JSR	Write_DBuffer   	; returns status in ACC

* Packet command and packet data were sent

* Give Busy a chance to go low
	MOV	#$FF,pcount
dp5:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,dp6		; Busy is Low
	JSR	WAIT100ms
	DEC	pcount
	BNE	dp5

	LDA	#$03			; Busy never went low
	STA	errlog
	RTS

dp6:	CLRA
	RTS
*==========================================================================
* Fast Forward
* Read ACC number of words, only the last one will be reported
FFORWARD:
      STA	wcount
ffbr: JSR   	READ_CD_WORD   	; read two bytes
      DEC	wcount
      BNE	ffbr
      RTS	  
*==========================================================================
* Flush any remaining data in the block (flush 2048 bytes to make sure)
FLUSH:	LDA	#$00
	JSR	FFORWARD
	LDA	#$00
	JSR	FFORWARD
	LDA	#$00
	JSR	FFORWARD
	LDA	#$00
	JSR	FFORWARD

	RTS
*==========================================================================
****************************************************************************
* Lower Level CDROM Drive Functions
****************************************************************************
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* CDROM subroutines
* Returns Alt_Status in status as well as ACC
* uses ACC, Xreg
* Alt_Status Register is $6
Get_Alt_Status:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$FE,PORTA	; register address $6
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set
	
	BCLR	CS1,PORTC	; /CS1 low

	BCLR	RD,PORTC	; /RD low

	NOP
	CLR	DDRA		; Port A Input

	LDA	PORTA
	STA	status

	BSET	RD,PORTC	; /RD high

	BSET	CS1,PORTC	; /CS1 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* PORTA is to be Input by default
* Read a selected CDROM Port
* port number in ACC
ReadPort:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	LDA	PortAddress
	AND	#$07		; address is low 3 bits
	ORA	#$F8		; keep everthing else high
	STA	PORTA		
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	RD,PORTC	; /RD low

	NOP
	CLR	DDRA		; Port A Input

	LDA	PORTA
	STA	status

	BSET	RD,PORTC	; /RD high

	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* Read CDROM Data port (word) and send it back to the PC
DoData1:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F8,PORTA	; address 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	CLR	DDRA		; Port A Input
	CLR	DDRB		; Port B Input

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	RD,PORTC	; /RD low

	NOP

	JSR	WAIT1ms
	LDA	PORTA
	JSR	SENDCHAR

	JSR	WAIT1ms
	LDA	PORTB
	JSR	SENDCHAR

	BSET	RD,PORTC	; /RD high

	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* Write ACC to CDROM Command Port
Write_Command:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$FF,PORTA	; address 7
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	WR,PORTC	; /WR low

	MOV	cmdsave,PORTA

	BSET	WR,PORTC	; /WR high
	BSET	CS0,PORTC	; /CS0 high

	CLR	DDRA		; Port A back to inputs

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* Write packet from buffer into CDROM data port
Write_DBuffer:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output
	MOV	#$FF,DDRB	; Port B Output

	MOV	#$F8,PORTA	; address 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	LDHX	#11
wdblp:	   LDA	buffer,x
	   STA	PORTA
	   DECX

	   LDA	buffer,x
	   STA	PORTB

	   BCLR	   CS0,PORTC	; /CS0 low
	   BCLR	   WR,PORTC	; /WR low
	   BSET	   WR,PORTC	; /WR high
	   BSET	   CS0,PORTC	; /CS0 high

	   DECX
	   BPL	wdblp


	CLR	DDRA		; Port A back to inputs
	CLR	DDRB		; Port B back to inputs

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
READ_CD_WORD:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F8,PORTA	; address 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	CLR	DDRA		; Port A inputs
	CLR	DDRB		; Port B inputs

	BCLR	CS0,PORTC	; /CS0 low
	BCLR	RD,PORTC	; /RD low

	LDA	PORTA
	STA	byte_low

	LDA	PORTB
	STA	byte_high

	BSET	RD,PORTC	; /RD high
	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
TestRD_CD:
	LDA	temp
	STA	byte_low
	INCA
	STA	byte_high	
	INCA
	STA	temp
	RTS
*============================================================================
* Returns Error Code in status as well as ACC
* uses ACC, Xreg
* Error Reg is b001
*    d7    d6   d5    d4    d3  d2  d1  d0 
*   /Res  /RD  /WR  /CS1  /CS0  A2  A1  A0
Get_Error_Rd:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F9,PORTA	; register $1		

	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	CLR	DDRA		; Port A Input

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	RD,PORTC	; /RD low

	NOP

	LDA	PORTA
	STA	status

	BSET	RD,PORTC	; /RD high

	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7    d6   d5    d4    d3  d2  d1  d0
*   /Res  /RD  /WR  /CS1  /CS0  A2  A1  A0
* Address from ACC, data from datasave
WritePort:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	LDA	PortAddress
	AND	#$07		; address is low 3 bits
	ORA	#$F8		; keep everthing else high
	STA	PORTA		
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	LDA	datasave
	STA	PORTA		; data

	BCLR	CS0,PORTC	; /CS0 low
	BCLR	WR,PORTC	; /WR low
	BSET	WR,PORTC	; /WR high
	BSET	CS0,PORTC	; /CS0 high

	CLR	DDRA		; Port A back to inputs

	CLI			; re-enable interrupts

	RTS
*==========================================================================
* Wait for CDROM ready
* Returns: ACC returns 0=ready, $FF=timed out
Wait_For_Ready:

	MOV	#30,wfrcount		; 30 seconds 

wfr1:		JSR	Get_Sense	; get the sense keys
		BEQ	wfr2		; sense code = 0

		LDA	#$80		; clear the first line
		JSR     CLR_DISP
		LDHX    #MSG7           ; 'Waiting For Ready'
		JSR     DOMSG

		JSR	WAIT250ms
		JSR	WAIT250ms

		JSR	Get_Sense	; get the sense keys
		BEQ	wfr2		; sense code = 0

		LDA	#$80		; clear the first line
		JSR     CLR_DISP
		JSR	WAIT250ms
		JSR	WAIT250ms

		DEC	wfrcount
		BNE	wfr1

	MOV	#$FF,wready_error		; we timed out

	LDA	#$80		; clear the first line
	JSR     CLR_DISP

	RTS

wfr2:	LDA	#$80		; clear the first line
	JSR     CLR_DISP

	CLRA				; unit ready
	STA	wready_error
	RTS
*==========================================================================
* Get the sense keys
Get_Sense:
	CLR	sense_error

* Form the command packet for TEST_UR
	CLRA
	LDHX	#11			; X= 11->0 is packet[0..11]
gs1:	   STA	  buffer,X
	   DECX
	   BPL    gs1
      LDA	#TEST_UR
      STA	buffer+11

* Send Packet Command
*    If ACC returns with $FE, BUSY never went low
*    Otherwise, ACC returns $00, with status register in status
	JSR	Packet_Command	     ; send Packet Command
	BEQ	gs2
	LDA	#$FE
	STA	sense_error
	RTS		        ; $FE = BUSY never went low, command not sent

* Give DRQ a chance to go high
gs2:	MOV	#$FF,pcount
gs3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,gs4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	gs3
	LDA	#$FD		; $FD = DRQ never went high
	STA	sense_error
	RTS

* Send the Packet
gs4:	JSR	Write_DBuffer

* Packet command and packet data were sent

* Give Busy a chance to go low
	MOV	#$FF,pcount
gs5:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,gs6		; Busy is Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	gs5

	LDA	#$FC			; Busy never went low
	STA	sense_error
	RTS

* Read the Error Register
gs6:	JSR	Get_Error_Rd	; Returns Error code in status as well as ACC
	LSRA
	LSRA
	LSRA
	LSRA
	STA	sense
	RTS			; returns sense bits in ACC: 3 2 1 0
*==========================================================================
* Wait for DRQ
Wait_For_DRQ:
	LDA	#$FF
	STA	pcount
wdbr1:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,wdbr2		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	wdbr1

	LDA	#$02			; DRQ never went high
	RTS
wdbr2:  CLRA
	RTS
*==========================================================================
******************************************************************************
* Functions for ISO-9660 CD Data Discs
******************************************************************************
*==========================================================================
* Read_LBA command Packet 
CMD_READ_LBA:
	CLR	sense_error

	MOV	#READ_DATA10,buffer+11	; op code		byte 0
	CLR	buffer+10		; Reserved		byte 1
	MOV	LBA_Adr+3,buffer+9	; LBA Byte 3		byte 2
	MOV	LBA_Adr+2,buffer+8	; LBA Byte 2		byte 3
	MOV	LBA_Adr+1,buffer+7	; LBA Byte 1		byte 4
	MOV	LBA_Adr+0,buffer+6	; LBA Byte 0		byte 5
	CLR	buffer+5		; Reserved		byte 6
	CLR	buffer+4		; Transfer Length MSB	byte 7
	MOV	#$01,buffer+3		; Transfer Length LSB	byte 8
	CLR	buffer+2		; Reserved		byte 9
	CLR	buffer+1		; Reserved		byte 10
	CLR	buffer+0		; Reserved		byte 11

        JSR	Do_Packet		; errlog in ACC

	RTS
*==========================================================================
* Initialize data buffer pointers
Init_BPtrs:
	CLR	LBA_Count
	CLR	LBA_Count+1
	CLR	CBUFF_Count
	CLR	CBUFF_In_Ptr
	CLR	CBUFF_Out_Ptr
	RTS
*---------------------------------------------------------
* Get LBA for the Root Directory 
* Root Directory LBA is at bytes 158-162
* Primary Descriptor Volume must already have been issued
Get_Dir_LBA:

* Sift through the data

* discard 158 bytes
	MOV	#79,vcount

gdbr1:  JSR     READ_CD_WORD		; discard it
	DEC	vcount
	BNE	gdbr1

	JSR     READ_CD_WORD		; Directory LBA

	MOV	byte_low,LBA_Adr+0
	MOV	byte_high,LBA_Adr+1

	JSR     READ_CD_WORD		; Directory LBA

	MOV	byte_low,LBA_Adr+2
	MOV	byte_high,LBA_Adr+3

	JSR	Flush
	RTS
*==========================================================================
*-----------------------------------------------------------------------------
* Get the File LBA
* We must have already found the Directory LBA and issued the Read LBA Command
Get_File:

* File Length of FILE to be found, length returned in flength
	JSR	File_Len

* Find the records
	MOV	#50,maxrecords

gfbr1:	JSR	Get_Dir_Record		; returns $FF if end of records
	BNE	gfbr2			; done, with no more records

* We have a file record
* Filename and LBA are in cbuffer [0..14] and [15..18]

* Convert the Filename to Upper case
*	JSR	UpperCase

* Compare the Filename with Requested Filename
	JSR	File_Comp

* If they match, then go to the next step, otherwise go again
	BEQ	gfbr3

	DEC	maxrecords       ; have we exceeded the max number of records?
	BNE	gfbr1 		 ; go again

* We have failed to find the file in the directory
gfbr2:	JSR	FLUSH		 ; flush any remaining data in the block 
	LDA	#$FF
	RTS

* We have a winner, File LBA is in LBA_Adr+n
gfbr3:

* Finally, instead of trying to calculate the number of bytes remaining 
*   we will just read and discard 2048 bytes
	JSR	FLUSH

	CLRA
	RTS
*==========================================================================
* File Length, length returned in flength
* string ends with '0'
File_Len:
        LDHX	#$0000
flbr1:  LDA	Fname,X
        BEQ	flbr2		; we are done
        INCX
        BRA	flbr1

flbr2:  DECX			; go back one
	STX	flength
        RTS
*==========================================================================
* Compare file names
* returns $FF if match, 0 if no match
File_Comp:
       	LDX	flength
	CLRH			; we are using X as an address pointer
fcbr1	LDA	Fname,X
	CMP	cbuffer,X
	BNE	fcbr2		; wrong
	DECX
	BPL	fcbr1

* we got here, so it is a match
	CLRA
	RTS

fcbr2:	LDA	#$FF
	RTS
*==========================================================================
* Convert to upper case, cbuffer, 14 characters
UpperCase:
	LDHX	#14
ucbr1:	LDA	cbuffer,X
	CMP	#$61
	BMI	ucbr2
	SUB	#$20
	STA	cbuffer,X
ucbr2:  DECX
	BPL	ucbr1
	RTS
*==========================================================================
* Use Our Filename
* Filename must be capital letters, terminated with 0.
FILENAME  FCB   'README.TXT',0
Our_Fname:
	LDHX	#15
ofnbr1:	LDA	FILENAME,X
	STA	Fname,X
	DECX
	BPL	ofnbr1
	RTS
*==========================================================================
*==========================================================================
* Get the Directory record
Get_Dir_Record:
Get2_Dir_Record:
* Start looking for the file
	CLRH				; we are using X as an address pointer

	JSR	READ_CD_WORD		; bytes 0,1			  W0
        MOV	byte_low,recordlength	; length of record (byte 0)
	BNE	ngdr1			; a length of 0 means no more records

	LDA	#$08		
	RTS

* Get the File LBA, if this is not the File it will be ignored
ngdr1:	JSR     READ_CD_WORD		; bytes 2,3			   W1

	MOV	byte_low,LBA_Adr+0
	MOV	byte_high,LBA_Adr+1

	JSR     READ_CD_WORD		; bytes 4,5			   W2

	MOV	byte_low,LBA_Adr+2
	MOV	byte_high,LBA_Adr+3

	JSR     READ_CD_WORD		; bytes 6,7			   W3
	JSR     READ_CD_WORD		; bytes 8,9			   W4

	JSR     READ_CD_WORD		; bytes 10,11			   W5
	MOV	byte_low,LBA_Length+0
	MOV	byte_high,LBA_Length+1

	JSR     READ_CD_WORD		; bytes 12,13			   W6
	MOV	byte_low,LBA_Length+2
	MOV	byte_high,LBA_Length+3

*   Fast Forward to byte 32 (file identifier length)  [word 16 - word 6]
	LDA	#10
	JSR	FFORWARD						;  W17

*   byte_low is the file name length
*   byte_high is the first character in the file name
	LDA	byte_high
	STA	cbuffer+0	; byte 33 (word 17)

*   Get 14 characters
	MOV	#$01,bindex		; buffer index

	MOV	#07,wcount

ngdr2:	  JSR     READ_CD_WORD					    ; W18-W25

	  LDX	bindex
	  CLRH
	  LDA	byte_low
	  STA	cbuffer,X

	  INCX

	  LDA	byte_high
	  STA	cbuffer,X
	  INCX

	  STX	bindex

	  DEC	wcount
	  BNE	ngdr2

* The file name is in cbuffer

* We are now at word 24, fast forward to the end of the record
	LDA	recordlength
	LSRA			; convert to words
	SUB	#24
	BEQ	ngdr3
	BMI	ngdr4		; less than or equal to 0
	JSR	FFORWARD

ngdr3:	CLRA
	RTS

ngdr4:	LDA	#$FE		; exceeded expected length of record
	RTS


* Start looking for the file
	CLRH				; we are using X as an address pointer

	JSR	READ_CD_WORD		; bytes 0,1
        MOV	byte_low,recordlength	; length of record (byte 0)
	BNE	ngdr1			; a length of 0 means no more records

	LDA	#$08		
	RTS
*==========================================================================
* Development Test
DIR_LBA:   FCB   $1b,$00,$00,$00	; least to most - Directory

DEV1:	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Start the Disc
	JSR	Start_Disc
	BEQ	dv1br1
	JMP	FLASH_ERR
dv1br1:	

* Wait_For_Ready
	JSR	Wait_For_Ready
	BEQ	dv1br2
	JMP	FLASH_ERR

* Initialize Buffer Pointers
dv1br2:	JSR	Init_BPtrs

* LBA Address of Directory
	LDX	#$0003
	CLRH
dv1br3:	LDA	DIR_LBA,X
	STA	LBA_Adr,X
	DECX
	BPL	dv1br3

* Issue the Read_LBA Command
	JSR	CMD_READ_LBA
	BEQ	dv1br4
	JMP	FLASH_ERR
dv1br4:	

	RTS
*==========================================================================
******************************************************************************
* Various Test Functions for the CDROM Drive
******************************************************************************
*==========================================================================
* Test Program for Getting Table of Contents(10)
TEST_TOC:
* Hard Reset
*	JSR	Hard_Reset

* Start the Disc
	JSR	Start_Disc
	BEQ	toct1
	JMP	FLASH_ERR

* Wait_For_Ready
toct1:	JSR	Wait_For_Ready
	BEQ	toct2
	JMP	FLASH_ERR

toct2:	JSR	Tst_Read_TOC10

	RTS
*====================================================
TTOC10:	FCB   $00,$00		; 11,10
	FCB   $80,$00		; 9,8
	FCB   $20,$01		; 7,6
	FCB   $00,$00		; 5,4
	FCB   $00,$10		; 3,2
	FCB   $02,READ_TOC	; 1,0

* Read Table of Contents 10 (for audio CD tracks)
Tst_Read_TOC10:

* Form the command packet for Read TOC 10
	LDHX	#11			; X= 11->0 is packet[0..11]
ttc1:	   LDA	  TTOC10,X
	   STA	  buffer,X
	   DECX
	   BPL    ttc1

      JSR	Do_Packet		; returns $00 = no errors
      BEQ	ttc2
      JMP	FLASH_ERR

* wait while BSY
ttc2:

* get and decode the data
	JSR     READ_CD_WORD	; length

	MOV	byte_low,length+1	; length MSB
	MOV	byte_high,length+0	; length LSB

	LDA	length+0	; subtract 4+11
	ADD	#$F1
	STA	length+0

	LDA	length+1
	ADC	#$FF
	STA	length+1

	JSR     READ_CD_WORD	; first, last session (ignore)

* There are 11 bytes/record
ttclp:	JSR     READ_CD_WORD	; Session #, ADR:Control (ignore)

	JSR     READ_CD_WORD	; TNO (ignore), Point

	LDA	byte_high	; 0-99 is track number
	STA	track
	CMP	#99
	BMI	ttc3	

* not a track number 
	JSR     READ_CD_WORD	; byte 4, 5
	JSR     READ_CD_WORD	; byte 6, 7
	JSR     READ_CD_WORD	; byte 8, 9 
	JSR     READ_CD_WORD    ; byte 10, 0 (byte_high is byte 0 next record)
	BRA	ttc_odd

* we have a track number
ttc3:
	JSR     READ_CD_WORD	; byte 3, byte 4 (ignore)
	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; M, S
	MOV	byte_low,Mval
	MOV	byte_high,Sval

	JSR     READ_CD_WORD	; F, first bute of next record
	MOV	byte_low,Fval

	JSR	Display_MSF

* byte_high is first byte of next record
ttc_odd:
	JSR	Do_Length
	BPL	ttc4		; msb less than zero means we are done

* we are done, so just read and discard 2048 bytes
	JSR	FLUSH
	RTS

* byte_high is byte 0 of next record, ignore it
ttc4:	JSR     READ_CD_WORD    ; byte 1, 2 (ignore)

	JSR     READ_CD_WORD	; byte 3 Point, byte 4 (ignore)

	LDA	byte_low	; byte 3: 0-99 is track number
	STA	track
	CMP	#99
	BMI	ttc5	

* not a track number 
	JSR     READ_CD_WORD	; byte 5, byte 6
	JSR     READ_CD_WORD	; byte 7, byte 8
	JSR     READ_CD_WORD	; byte 9, byte 10
	JMP	ttclp		; we are at an even boundary again

ttc5:	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; byte 7, byte 8 (Mvalue)
	MOV	byte_high,Mval

	JSR     READ_CD_WORD	; S, F
	MOV	byte_low,Sval

	MOV	byte_high,Fval

	JSR	Display_MSF

	JSR	Do_Length
	BMI	ttc6		; msb less than zero means we are done
	JMP	ttclp

*  we will just read and discard 2048 bytes
ttc6:	JSR	FLUSH
	RTS
*==========================================================================
Do_Length:
	LDA	length		; subtract 11
	ADD	#$F5
	STA	length
	LDA	length+1
	ADC	#$FF
	STA	length+1
	RTS
*---------------------------------------------------------------------------
* displayed in hex, not BCD
Display_MSF:
	LDA	track
        LDX     #$80    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	Mval
        LDX     #$84    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	Sval
        LDX     #$87    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	Fval
        LDX     #$8A    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

*	LDA	length
*	LDX     #$C0    	; set cursor location
* 	JSR     DISPLAY_BINARY  ; Display Binary from ACC

*	LDA	length+1
*	LDX     #$C4    	; set cursor location
*	JSR     DISPLAY_BINARY  ; Display Binary from ACC

	JSR	WAIT1s
	RTS
*==========================================================================
TEST_PLAYMSF:

* Start the Disc
	JSR	Start_Disc
	BEQ	tpm1
	JMP	FLASH_ERR

* Wait_For_Ready
tpm1:	JSR	Wait_For_Ready
	BEQ	tpm2
	JMP	FLASH_ERR

tpm2:	JSR	Test_Play_MSF

	JSR	Get_Alt_Status
	LDX	#$D0
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	JSR	WAIT250ms

	RTS
*====================================================
* 	       M   F    S   (BCD)
PMSF:	FCB   00,02,32	; track 1
	FCB   01,32,45	; track 2
	FCB   03,10,45	; track 3
	FCB   37,07,45	; track 4
	FCB   38,41,45	; track 5

EM	EQU	99	; End M
ES	EQU	59	; End S
EF	EQU	74	; End F

* Play Audio MSF from Table
Test_Play_MSF:

* Form the command packet for PLAY_AUDIO_MSF
	LDX	#$0C		; track number-1
	STX	track
	CLRH			; we are using X as an address pointer

        MOV	#PLAY_AUDIO_MSF,buffer+11	; byte 0	Op Code
	CLR	buffer+10	; byte 1
	CLR	buffer+9	; byte 2

	LDA	PMSF,X
	STA	buffer+8	; byte 3	Start M
	STA	Mval

	INCX
	LDA	PMSF,X
	STA	buffer+7	; byte 4	Start S
	STA	Sval

	INCX
	LDA	PMSF,X
	STA	buffer+6	; byte 5	Start F
	STA	Fval

	MOV	#EM,buffer+5	; byte 6	Ending M
	MOV	#ES,buffer+4	; byte 7	Ending S
	MOV	#EF,buffer+3	; byte 8	Ending F
	CLR	buffer+2	; byte 9
	CLR	buffer+1	; byte 10
	CLR	buffer+0	; byte 11

	JSR	Do_Packet	; returns $00 = no errors
	BEQ	pm2
	JMP	FLASH_ERR

* done
pm2:	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	Display_Buffer
	RTS
*---------------------------------------------------------------------------
* Display the data buffer
Display_Buffer:
	LDA	buffer+11
        LDX     #$80    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+10
        LDX     #$83    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+9
        LDX     #$86    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+8
        LDX     #$89    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+7
        LDX     #$8C    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+6
        LDX     #$8F    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+5
        LDX     #$92    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+4
        LDX     #$C0    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+3
        LDX     #$C3    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+2
        LDX     #$C6    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+1
        LDX     #$C9    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+01
        LDX     #$CC    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	RTS
*=============================================================================
*==========================================================================
* Show Filename
* Filename must be capital letters, terminated with 0.
Show_Fname:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$80
        JSR     lcd_wr_instr

	LDHX	#fname
sfnbr: 	LDA     ,X
        BEQ     sfnout          ; we are done

        JSR     lcd_wr_data     ; put out the character (does not use XReg)
	AIX	#$01
        BRA     sfnbr

sfnout:	RTS
*==========================================================================
*==========================================================================
Show_LBA:
	LDA	LBA_Adr+3
	LDX	#$80
	CLRH
	JSR	DISPLAY_BINARY

	LDA	LBA_Adr+2
	LDX	#$83
	CLRH
	JSR	DISPLAY_BINARY

	LDA	LBA_Adr+1
	LDX	#$86
	CLRH
	JSR	DISPLAY_BINARY

	LDA	LBA_Adr+0
	LDX	#$89
	CLRH
	JSR	DISPLAY_BINARY

	RTS
*=============================================================================
* Send data to the lcd
Show_Data:

	LDA	#$C0
        JSR     CLR_DISP

	LDA	LBA_Adr+3
        LDX     #$C0   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Adr+2
        LDX     #$C2   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Adr+1
        LDX     #$C4   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Adr+0
        LDX     #$C6   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Length+3
        LDX     #$CA   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Length+2
        LDX     #$CC   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Length+1
        LDX     #$CE   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	LBA_Length+0
        LDX     #$D0   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	RTS
*=============================================================================
* Check the Dump Switch
DSWITCH:
	BCLR	SWEN,PORTC
	LDA	PORTA
	BSET	SWEN,PORTC
	AND	#$20
	RTS
*==========================================================================
Add_Word:
	PSHH			; save H Register

	JSR	Wait_For_DRQ
	JSR	READ_CD_WORD

	CLRH
	LDX	CBUFF_In_Ptr

	LDA	byte_low
	STA	cbuffer,X

	INCX
	
	LDA	byte_high
	STA	cbuffer,X

	LDA	CBUFF_In_Ptr
	ADD	#$02
	AND	#$3F		; 64 bytes max, for now
	STA	CBUFF_In_Ptr

	INC	CBUFF_Count
	INC	CBUFF_Count

	PULH			; restore H Register
	RTS
*==========================================================================
*============================================================================
* Convert binary to bcd: 99,999. max
* uses ACC, Xreg, binl, binh, bcd0,bcd1,bcd2
* input from binl,binh, output in bcd0,bcd1,bcd2
BIN_6BCD:
        CLR     bcd0
        CLR     bcd1
        CLR     bcd2

        LDX     #16
bcd6lp:
        ROL     binl
        ROL     binh

	LDA	bcd0
	ADC	bcd0
	DAA
	STA	bcd0

	LDA	bcd1
	ADC	bcd1
	DAA
	STA	bcd1

	LDA	bcd2
	ADC	bcd2
	DAA
	STA	bcd2

        DECX
        BNE     bcd6lp

        RTS
*==========================================================================
* Display BDC values
Display_6BCD:

	CLRH
	LDA	bcd2
	BEQ	dbcd1
        LDX     #$C0    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

dbcd1:	CLRH
	LDA	bcd1
	BEQ	dbcd2
        LDX     #$C2    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

dbcd2:	CLRH
	LDA	bcd0
        LDX     #$C4    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	RTS
*===========================================================================
*===========================================================================
* Convert binary to bcd, 99. max
* uses ACC, Xreg, binl, bcd0
* input from ACC, output in bcd0
BIN_BCD_99:
        STA     binl
        CLR     bcd0
        LDX     #8
bcdlp:
	LSL	binl
	LDA	bcd0
	ADC	bcd0
	DAA
	STA	bcd0

abr1:   DECX
        BNE     bcdlp

        RTS
*============================================================================
MSG0:   FCB  $80,'    PSP12       ',$FF
MSG1:   FCB  $C0,'CDROM Controller',$FF
MSG2:   FCB  $80,'Copyright 1999  ',$FF
MSG3:   FCB  $C0,' Jed Margolin   ',$FF
MSG4:   FCB  $C1,'               ',$FF
MSG5:   FCB  $80,'   FLASH!      ',$FF
MSG6:   FCB  $80,'Error',$FF
MSG7:   FCB  $80,'Waiting For Ready ',$FF
MSG8:   FCB  $80,'Ready             ',$FF
MSG9:   FCB  $80,'Serial Port Mode  ',$FF
*==========================================================================
MMSG0:  FCB  $80,'1=Play CD',$FF
MMSG1:  FCB  $C0,'2=Drive Info',$FF
MMSG2:  FCB  $8D,'3=SCode',$FF
MMSG3:  FCB  $CD,'4=Test',$FF
MMSG4:  FCB  $80,'$FFF6-$FFFD',$FF
*==========================================================================
* Display Menu Messages
Display_Men_Msgs:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

        LDHX    #MMSG0 		; '1=Play CD'
        JSR     DOMSG

        LDHX    #MMSG1 		; '2=Drive Info'
        JSR     DOMSG

        LDHX    #MMSG2 		; '3=SCode'
        JSR     DOMSG

        LDHX    #MMSG3 		; '4=Test'
        JSR     DOMSG

	RTS
*===========================================================================
*===========================================================================
****************************************************************************
* Switch Functions
****************************************************************************
*===========================================================================
* Initialize the Switch data
INIT_SWITCHES:
	LDA	#$FF
	STA	sw1
	STA	sw2
	STA	sw3
	STA	sw4
	STA	sw5
	STA	sw6
	
	LDA	#$03
	STA	sw1+1
	STA	sw2+1
	STA	sw3+1
	STA	sw4+1
	STA	sw5+1
	STA	sw6+1

	RTS
*===========================================================================
* Read and debounce the switches during Timer Interrupts
DO_SWITCHES:

* Because we are using Port C and Port A, every other function that uses them
* *must* disable interrupts while using them.

* Read the switches
	MOV	#$FF,PORTC	; IDE Interface deselected
	CLR	DDRA		; Port A Inputs

	BCLR	SWEN,PORTC	; Switches selected
        LDA     PORTA
	BSET	SWEN,PORTC	; Switches deselected
	STA	swtemp		; switch: 0 = on

* sw1 =debounce register
* sw1+1: d2=positive transition, d1=old state, d0=debounced state, 
	LDHX	#sw1		
	BSR	swsub		; switch 1	

	LDHX	#sw2		
	BSR	swsub		; switch 2

	LDHX	#sw3		
	BSR	swsub		; switch 3

	LDHX	#sw4		
	BSR	swsub		; switch 4

	LDHX	#sw5		
	BSR	swsub		; switch 5

	LDHX	#sw6		
	BSR	swsub		; switch 6

	RTS
*----------------------------------
* debounce switch, check for positive transition 
swsub:
* get the two switch variables (it's faster this way)
	LDA	,X		
	STA	swtemp+1

	LDA	1,X
	STA	swtemp+2

* look at the switch
	ROR	swtemp		; switch state is in Carry
        ROL     swtemp+1	; debounce register for the particular switch
	BEQ	sws1		; all zero

	LDA	swtemp+1	; debounce register
	CMP	#$FF
	BNE	swsout		; not all zeros, not all ones: do nothing

	BSET	0,swtemp+2	; all ones: set swtemp+2(bit 0)
	BRA	sws2

sws1:	BCLR	0,swtemp+2	; all zeros: clear bit 0, swtemp+2

* switch is debounced, state is in swtemp+2(d0)
sws2:	BRSET	0,swtemp+2,sws3	; switch not pressed, update old switch state

* Switch is pressed, is it a positive transition?

* If the old switch was pressed, it is not a transistion
	BRCLR	1,swtemp+2,swsout	

* We have a winner, set the positive transition flag
* (Whoever uses the flag is responsible for clearing it)
	BSET	2,swtemp+2

* put new state into old state
sws3:   BRSET	0,swtemp+2,sws4
	BCLR	1,swtemp+2		
	BRA	swsout

sws4:	BSET	1,swtemp+2

* Write switch parameters back to the current switch
swsout:	LDA	swtemp+1		
	STA	,X

	LDA	swtemp+2
	STA	1,X

	RTS
*====================================================
* Look for switch transitions for changing the track number
SWTRANS:

* increment newtrack?
	BRCLR	2,sw4+1,csbr2		; switch: track +
	BCLR	2,sw4+1

	BSET	7,chtrack		; we have changed the track

	LDA	newtrack
	ADD	#$01
	CMP	track_count
	BLS	csbr1
	LDA	#$01	
csbr1:	STA	newtrack
	RTS

* decrement newtrack?
csbr2:	BRCLR	2,sw3+1,csbr4		; switch: track -
	BCLR	2,sw3+1

	BSET	7,chtrack	; we have changed the track

	LDA	newtrack
	SUB	#$01
	BNE	csbr3
	LDA	track_count
csbr3:	STA	newtrack

csbr4:  RTS
*====================================================
* Test Switches - On/Off
TEST_SW_A:
	LDA	#$80		; clear line 1
        JSR     CLR_DISP

	LDA	#$C0		; clear line 2
        JSR     CLR_DISP

	JSR	INIT_SWITCHES

tsw1lp:	JSR	DO_SWITCHES

	JSR	Display_Switch1		; display switch state

        JSR     WAIT10ms

	BRA	tsw1lp
*============================================================================
* Test Switches - Positive Transitions
TEST_SW_B:

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	INIT_SWITCHES

	CLR	pcount

tsw2lp:	JSR	DO_SWITCHES

	JSR	Display_Switch2		; positive switch transition

        JSR     WAIT10ms

	JSR	SWTRANS_tst		; do the switch transitions

	BRA	tsw2lp
*====================================================
* Display switch state (6 switches)
Display_Switch1		
	LDA	#$80
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw1+1,dswbr1
	LDA	#'0'
dswbr1: JSR     lcd_wr_data     ; put out the character

	LDA	#$83
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw2+1,dswbr2
	LDA	#'0'
dswbr2: JSR     lcd_wr_data     ; put out the character

	LDA	#$86
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw3+1,dswbr3
	LDA	#'0'
dswbr3: JSR     lcd_wr_data     ; put out the character

	LDA	#$89
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw4+1,dswbr4
	LDA	#'0'
dswbr4: JSR     lcd_wr_data     ; put out the character

	LDA	#$8C
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw5+1,dswbr5
	LDA	#'0'
dswbr5: JSR     lcd_wr_data     ; put out the character

	LDA	#$8F
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw6+1,dswbr6
	LDA	#'0'
dswbr6: JSR     lcd_wr_data     ; put out the character

	RTS
*===========================================================================
* Display positive switch transitions
Display_Switch2		
	LDA	#$80
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw1+1,dstbr1
	LDA	#'0'
dstbr1: JSR     lcd_wr_data     ; put out the character

	LDA	#$83
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw2+1,dstbr2
	LDA	#'0'
dstbr2: JSR     lcd_wr_data     ; put out the character

	LDA	#$86
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw3+1,dstbr3
	LDA	#'0'
dstbr3: JSR     lcd_wr_data     ; put out the character

	LDA	#$89
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw4+1,dstbr4
	LDA	#'0'
dstbr4: JSR     lcd_wr_data     ; put out the character

	LDA	#$8C
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw5+1,dstbr5
	LDA	#'0'
dstbr5: JSR     lcd_wr_data     ; put out the character

	LDA	#$8F
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw6+1,dstbr6
	LDA	#'0'
dstbr6: JSR     lcd_wr_data     ; put out the character

	RTS
*===================================================
* for testing Switch transitions
SWTRANS_tst:
	TST	pcount
	BEQ	swtbr1

	DEC	pcount
	BNE	swtbr3

* clear all the transition flags
	BCLR	2,sw1+1
	BCLR	2,sw2+1
	BCLR	2,sw3+1
	BCLR	2,sw4+1
	BCLR	2,sw5+1
	BCLR	2,sw6+1
	RTS

* look for a transition
swtbr1:	BRSET	2,sw1+1,swtbr2
	BRSET	2,sw2+1,swtbr2
	BRSET	2,sw3+1,swtbr2
	BRSET	2,sw4+1,swtbr2
	BRSET	2,sw5+1,swtbr2
	BRSET	2,sw6+1,swtbr2
	RTS

* we had a transition, start the counter
swtbr2:	MOV	#40,pcount

swtbr3:	RTS
*===================================================
* Timer 1 Overflow Interrupt (USER_T1_OF_VEC)
TIRQ:	PSHH			; save H Register

	JSR	DO_SWITCHES	; read and debounce

	JSR	SWTRANS

	MOV	#$FF,irqflag

	BCLR	TOF,T1SC	; reset interrupt flag

TIRQRET:
	PULH			; restore H Register
	RTI
*=============================================================================
*=============================================================================
